from wxPython.wx import *
import IMCom

IM_Disconnected   = 0
IM_PresenceUpdate = 1
IM_MessageReceive = 2
IM_MessageError   = 3
IM_InfoError      = 4
IM_FileReceive    = 5
IM_FileReceived   = 6
IM_FileError      = 7
IM_AgentList      = 8
IM_AgentRegister  = 9
IM_Subscribe      = 10
IM_Subscribed     = 11
IM_Unsubscribe    = 12
IM_Unsubscribed   = 13
IM_RosterUpdateCheck = 14
IM_RosterUpdate      = 15
IM_VCard             = 16
IM_NoVCard           = 17
IM_GrabRoster        = 18
IM_Login             = 19
IM_AgentRegistered   = 20
IM_IQError           = 21

myEVT_IMCom = wxNewEventType()
IMCOM_ID = 4999
def EVT_IMCOM(win, id, func):
    win.Connect(id, -1, myEVT_IMCom, func)

class IMComEvent(wxPyCommandEvent):
    def __init__(self, evtType, id):
        wxPyCommandEvent.__init__(self,evtType, id)
        self.type = None
        self.data = None

    def setStuff(self, type, dict):
        self.type = type
        self.data = dict

class GUIBackendUI(wxApp):
    def OnInit(self):
        frame = wxFrame(None, -1, "Dummy window")
        frame.Show(true)
        self.SetTopWindow(frame)

class GUIBackend:
    def __init__(self, handler):
        # do initialization of the IMCom library here.
        self.imcom = None
        self.handlerlist = []
        if(handler == None):
            self.handlerlist.append(self)
        else:
            self.handlerlist.append(handler)
        EVT_IMCOM(handler, IMCOM_ID, handler.IMComEventHandler)

    def addHandler(self, object, function):
        self.handlerlist.append(object)
        EVT_IMCOM(object, IMCOM_ID, function)

    def removeHandler(self, object):
        try:
            self.handlerlist.remove(object)
        except:
            pass

    def sendMessage(self, msg):
        for item in self.handlerlist:
            wxPostEvent(item, msg)

    def sendLogin(self, server, port, user, password, resource, ssl):
        #self.imcom.changeProfile("floobin.cx", 5222, "test2",
        #                         "2gwen2", "testing gui")
        self.imcom = IMCom.IMCom(self)
        self.imcom.changeProfile(server, port, user, password, resource, ssl)
        return true

    def IMComEventHandler(self, ie):
        print "Received an ", ie.type, " containing data: ", ie.data

    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    # -------------------------------------------------------------------------
    #  The functions which begin with handle are callbacks from the
    #  IMCom library
    # -------------------------------------------------------------------------
    # +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    def login(self, successful):
        data = {}
        data['successful'] = successful
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_Login, data)
        self.sendMessage(ie)

    def grabRoster(self):
        data = {}
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_GrabRoster, data)
        self.sendMessage(ie)

    def handleDisconnected(self):
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_Disconnected, {})
        self.sendMessage(ie)


    def handlePresenceUpdate(self, who, show, status, available, resource,
        dup):
        data = {}
        data['who'] = who
        data['show'] = show
        data['status'] = status
        data['available'] = available
        data['resource'] = resource
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_PresenceUpdate, data)
        self.sendMessage(ie)


    def handleMessageReceive(self, ffrom, body, thread, delay, resource):
        data = {}
        data['ffrom'] = ffrom
        data['body'] = body
        data['thread'] = thread
        data['delay'] = delay
        data['resource'] = resource
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_MessageReceive, data)
        self.sendMessage(ie)

    def handleMessageError(self, ffrom, body, code):
        data = {}
        data['ffrom'] = ffrom
        data['body'] = body
        data['code'] = code
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_MessageError, data)
        self.sendMessage(ie)


    def handleInfoError(self, code, text):
        data = {}
        data['code'] = code
        data['text'] = text
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_InfoError, data)
        self.sendMessage(ie)

    def handleIQError(self, ffrom, code, body):
        data = {}
        data['ffrom'] = ffrom
        data['code'] = code
        data['body'] = body
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_IQError, data)
        self.sendMessage(ie)

    def handleFileReceive(self, jid, url):
        data = {}
        data['jid'] = jid
        data['url'] = url
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_FileReceive, data)
        self.sendMessage(ie)


    def handleFileReceived(self, jid, file):
        data = {}
        data['jid'] = jid
        data['file'] = file
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_FileReceived, data)
        self.sendMessage(ie)


    def handleFileErrorReceived(self, jid, url, type, text):
        data = {}
        data['jid'] = jid
        data['url'] = url
        data['type'] = type
        data['text'] = text
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_FileError, data)
        self.sendMessage(ie)


    def handleAgentList(self, theList):
        data = {}
        data['theList'] = theList
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_AgentList, data)
        self.sendMessage(ie)


    def handleAgentRegister(self, id, ffrom, instructions, fields):
        data = {}
        data['id'] = id
        data['ffrom'] = ffrom
        data['instructions'] = instructions
        data['fields'] = fields
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_AgentRegister, data)
        self.sendMessage(ie)


    def handleAgentRegistered(self, ffrom):
        data = {}
        data['ffrom'] = ffrom
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_AgentRegistered, data)
        self.sendMessage(ie)


    def handleSubscribe(self, ffrom):
        data = {}
        data['ffrom'] = ffrom
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_Subscribe, data)
        self.sendMessage(ie)


    def handleSubscribed(self, ffrom):
        data = {}
        data['ffrom'] = ffrom
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_Subscribed, data)
        self.sendMessage(ie)

    def handleUnsubscribed(self, ffrom, success):
        data = {}
        data['ffrom'] = ffrom
        data['success'] = success
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_Unsubscribed, data)
        self.sendMessage(ie)


    def handleUnsubscribe(self, ffrom):
        data = {}
        data['ffrom'] = ffrom
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_Unsubscribed, data)
        self.sendMessage(ie)


    # Reports whether a change to the roster was successful or not
    def handleRosterUpdateCheck(self, successful, error):
        data = {}
        data['successful'] = successful
        data['error'] = error
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_RosterUpdateCheck, data)
        self.sendMessage(ie)


    # Called when there has been a change to the roster
    def handleRosterUpdate(self, jid, name, subs, success):
        data = {}
        data['jid'] = jid
        data['name'] = name
        data['subs'] = subs
        data['success'] = success
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_RosterUpdate, data)
        self.sendMessage(ie)


    def handleVCard(self, ffrom, fn, given, family, nickname):
        data = {}
        data['ffrom'] = ffrom
        data['fn'] = fn
        data['given'] = given
        data['family'] = family
        data['nickname'] = nickname
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_VCard, data)
        self.sendMessage(ie)


    def handleNoVCard(self, ffrom):
        data = {}
        data['ffrom'] = ffrom
        ie = IMComEvent(myEVT_IMCom, IMCOM_ID)
        ie.setStuff(IM_NoVCard, data)
        self.sendMessage(ie)

    def handleIdleTick( self ):
        # do nothing for now
        pass
        return


if __name__ == '__main__':
    app = GUIBackendUI(0)
    gb = GUIBackend(None)
    app.MainLoop()
